"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("..");
var volume_1 = require("memfs/src/volume");
var fs = require("fs");
describe('union', function () {
    describe('Union', function () {
        describe('sync methods', function () {
            it('Basic one file system', function () {
                var vol = volume_1.Volume.fromJSON({ '/foo': 'bar' });
                var ufs = new __1.Union;
                ufs.use(vol);
                expect(ufs.readFileSync('/foo', 'utf8')).toBe('bar');
            });
            it('File not found', function () {
                var vol = volume_1.Volume.fromJSON({ '/foo': 'bar' });
                var ufs = new __1.Union;
                ufs.use(vol);
                try {
                    ufs.readFileSync('/not-found', 'utf8');
                    throw Error('This should not throw');
                }
                catch (err) {
                    expect(err.code).toBe('ENOENT');
                }
            });
            it('Method does not exist', function () {
                var vol = volume_1.Volume.fromJSON({ '/foo': 'bar' });
                var ufs = new __1.Union;
                vol.readFileSync = undefined;
                ufs.use(vol);
                try {
                    ufs.readFileSync('/foo', 'utf8');
                    throw Error('not_this');
                }
                catch (err) {
                    expect(err.message).not.toBe('not_this');
                }
            });
            describe('existsSync()', function () {
                it('finds file on real file system', function () {
                    var ufs = new __1.Union;
                    ufs
                        .use(fs)
                        .use(volume_1.Volume.fromJSON({ "foo.js": "" }, "/tmp"));
                    expect(ufs.existsSync(__filename)).toBe(true);
                    expect(fs.existsSync(__filename)).toBe(true);
                    expect(ufs.existsSync("/tmp/foo.js")).toBe(true);
                });
            });
        });
        describe('async methods', function () {
            it('Basic one file system', function (done) {
                var vol = volume_1.Volume.fromJSON({ '/foo': 'bar' });
                var ufs = new __1.Union;
                ufs.use(vol);
                ufs.readFile('/foo', 'utf8', function (err, data) {
                    expect(err).toBe(null);
                    expect(data).toBe('bar');
                    done();
                });
            });
            it('File not found', function (done) {
                var vol = volume_1.Volume.fromJSON({ '/foo': 'bar' });
                var ufs = new __1.Union;
                ufs.use(vol);
                ufs.readFile('/not-found', 'utf8', function (err, data) {
                    expect(err.code).toBe('ENOENT');
                    done();
                });
            });
            it('No callback provided', function () {
                var vol = volume_1.Volume.fromJSON({ '/foo': 'bar' });
                var ufs = new __1.Union;
                ufs.use(vol);
                try {
                    ufs.stat('/foo2', 'utf8');
                }
                catch (err) {
                    expect(err).toBeInstanceOf(TypeError);
                }
            });
            it('No file systems attached', function (done) {
                var ufs = new __1.Union;
                ufs.stat('/foo2', 'utf8', function (err, data) {
                    expect(err.message).toBe('No file systems attached.');
                    done();
                });
            });
        });
    });
});
