"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.logErrAndExit = logErrAndExit;
exports.CommandRunner = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _serializeError() {
  const data = require("serialize-error");

  _serializeError = function () {
    return data;
  };

  return data;
}

function _ink() {
  const data = require("ink");

  _ink = function () {
    return data;
  };

  return data;
}

function _consumer() {
  const data = require("../api/consumer");

  _consumer = function () {
    return data;
  };

  return data;
}

function _defaultErrorHandler() {
  const data = _interopRequireDefault(require("./default-error-handler"));

  _defaultErrorHandler = function () {
    return data;
  };

  return data;
}

function _utils() {
  const data = require("../utils");

  _utils = function () {
    return data;
  };

  return data;
}

function _loader() {
  const data = _interopRequireDefault(require("./loader"));

  _loader = function () {
    return data;
  };

  return data;
}

function _logger() {
  const data = _interopRequireDefault(require("../logger/logger"));

  _logger = function () {
    return data;
  };

  return data;
}

class CommandRunner {
  constructor(command, args, flags) {
    this.command = command;
    this.args = args;
    this.flags = flags;
  }
  /**
   * run command using one of the handler, "json"/"report"/"render". once done, exit the process.
   */


  runCommand() {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      try {
        yield _this.runMigrateIfNeeded();

        if (_this.flags.json) {
          return yield _this.runJsonHandler();
        }

        if (_this.shouldRunRender()) {
          return yield _this.runRenderHandler();
        }

        if (_this.command.report) {
          return yield _this.runReportHandler();
        }
      } catch (err) {
        return _this.handleError(err);
      }

      throw new Error(`command "${_this.command.name}" doesn't implement "render" nor "report" methods`);
    })();
  }
  /**
   * when both "render" and "report" were implemented, check whether it's a terminal.
   * if it's a terminal, use "render", if not, use "report" because "report" is just a string
   */


  shouldRunRender() {
    const isTerminal = process.stdout.isTTY;

    if (this.command.report && !isTerminal) {
      return false;
    }

    return Boolean(this.command.render);
  }

  handleError(err) {
    _logger().default.error(`got an error from command ${this.command.name}: ${err}. Error serialized: ${JSON.stringify(err, Object.getOwnPropertyNames(err))}`);

    _loader().default.off();

    const errorHandled = (0, _defaultErrorHandler().default)(err);
    if (this.command.private) return serializeErrAndExit(err, this.command.name);
    return logErrAndExit(errorHandled || err, this.command.name);
  }

  runJsonHandler() {
    var _this2 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this2.flags.json) return null;
      if (!_this2.command.json) throw new Error(`command "${_this2.command.name}" doesn't implement "json" method`);
      const result = yield _this2.command.json(_this2.args, _this2.flags);

      _loader().default.off();

      const code = result.code || 0;
      const data = result.data || result;
      return _this2.writeAndExit(JSON.stringify(data, null, 2), code);
    })();
  }

  runRenderHandler() {
    var _this3 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this3.command.render) throw new Error('runRenderHandler expects command.render to be implemented');
      const result = yield _this3.command.render(_this3.args, _this3.flags);

      _loader().default.off();

      const {
        waitUntilExit
      } = (0, _ink().render)(result);
      yield waitUntilExit();
      return _logger().default.exitAfterFlush(result.props.code, _this3.command.name);
    })();
  }

  runReportHandler() {
    var _this4 = this;

    return (0, _bluebird().coroutine)(function* () {
      if (!_this4.command.report) throw new Error('runReportHandler expects command.report to be implemented');
      const result = yield _this4.command.report(_this4.args, _this4.flags);

      _loader().default.off();

      return _this4.writeAndExit(`${result.data}\n`, result.code);
    })();
  }

  writeAndExit(data, exitCode) {
    return process.stdout.write(data, () => _logger().default.exitAfterFlush(exitCode, this.command.name));
  }

  runMigrateIfNeeded() {
    var _this5 = this;

    return (0, _bluebird().coroutine)(function* () {
      // @ts-ignore LegacyCommandAdapter has .migration
      if (_this5.command.migration) {
        _logger().default.debug('Checking if a migration is needed'); // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!


        return (0, _consumer().migrate)(null, false);
      }

      return null;
    })();
  }

}

exports.CommandRunner = CommandRunner;

function serializeErrAndExit(err, commandName) {
  process.stderr.write((0, _utils().packCommand)((0, _utils().buildCommandMessage)((0, _serializeError().serializeError)(err), undefined, false), false, false));
  const code = err.code && (0, _utils().isNumeric)(err.code) ? err.code : 1;
  return _logger().default.exitAfterFlush(code, commandName);
}

function logErrAndExit(msg, commandName) {
  // @ts-ignore AUTO-ADDED-AFTER-MIGRATION-PLEASE-FIX!
  if (msg.code) throw msg;
  console.error(msg); // eslint-disable-line

  _logger().default.exitAfterFlush(1, commandName);
}