"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.BuilderService = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

class BuilderService {
  constructor(isolator, workspace) {
    this.isolator = isolator;
    this.workspace = workspace;
  }
  /**
   * runs a pipeline of tasks on all components in the execution context.
   */


  run(context) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      // make build pipe accessible throughout the context.
      const buildPipe = context.env.getPipe(context);

      if (!buildPipe) {
        throw new Error(`Builder service expects ${context.id} to implement getPipe()`);
      }

      const buildContext = Object.assign(context, {
        capsuleGraph: yield _this.isolator.createNetworkFromConsumer(context.components.map(component => component.id.toString()), _this.workspace.consumer)
      });
      const results = yield buildPipe.execute(buildContext);
      return {
        id: context.id,
        results
      };
    })();
  }

}

exports.BuilderService = BuilderService;