"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Packer = void 0;

function _bluebird() {
  const data = require("bluebird");

  _bluebird = function () {
    return data;
  };

  return data;
}

function _path() {
  const data = _interopRequireDefault(require("path"));

  _path = function () {
    return data;
  };

  return data;
}

function _execa() {
  const data = _interopRequireDefault(require("execa"));

  _execa = function () {
    return data;
  };

  return data;
}

function _fsExtra() {
  const data = _interopRequireDefault(require("fs-extra"));

  _fsExtra = function () {
    return data;
  };

  return data;
}

function _scope() {
  const data = _interopRequireDefault(require("../../scope/scope"));

  _scope = function () {
    return data;
  };

  return data;
}

function _environment() {
  const data = _interopRequireDefault(require("../../environment"));

  _environment = function () {
    return data;
  };

  return data;
}

function _scopeNotFound() {
  const data = require("./exceptions/scope-not-found");

  _scopeNotFound = function () {
    return data;
  };

  return data;
}

// @ts-ignore (for some reason the tsc -w not found this)
class Packer {
  constructor(scope) {
    this.scope = scope;
  }

  packComponent(componentId, scopePath, outDir, prefix = false, override = false, keep = false) {
    var _this = this;

    return (0, _bluebird().coroutine)(function* () {
      const scope = scopePath ? yield _scope().default.load(scopePath) : _this.scope;

      if (!scope) {
        throw new (_scopeNotFound().ScopeNotFound)(scopePath);
      } // TODO: change to use the new capsule


      const isolatedEnvironment = new (_environment().default)(scope, undefined);
      yield isolatedEnvironment.create();
      const isolatePath = isolatedEnvironment.path;
      const isolateOpts = {
        writeBitDependencies: true,
        createNpmLinkFiles: true,
        installPackages: false,
        noPackageJson: false,
        excludeRegistryPrefix: !prefix,
        saveDependenciesAsComponents: false
      };
      yield isolatedEnvironment.isolateComponent(componentId, isolateOpts);
      outDir = outDir || isolatePath;
      const packResult = yield npmPack(isolatePath, outDir, override);

      if (!keep) {
        yield isolatedEnvironment.destroy();
      }

      return packResult;
    })();
  }

}

exports.Packer = Packer;

function readPackageJson(outDir) {
  const pkgJson = _fsExtra().default.readJsonSync(_path().default.join(outDir, 'package.json'));

  return pkgJson;
}

function npmPack(_x, _x2) {
  return _npmPack.apply(this, arguments);
}

function _npmPack() {
  _npmPack = (0, _bluebird().coroutine)(function* (cwd, outputPath, override = false) {
    const result = yield (0, _execa().default)('npm', ['pack'], {
      cwd
    });
    const stdout = result.stdout;
    const tgzName = stdout.trim();

    const tgzOriginPath = _path().default.join(cwd, tgzName);

    const pkgJson = readPackageJson(cwd);

    const tarPath = _path().default.join(outputPath, tgzName);

    const response = {
      pkgJson,
      tarPath
    };

    if (tgzOriginPath !== tarPath && _fsExtra().default.pathExistsSync(tarPath)) {
      if (override) {
        _fsExtra().default.removeSync(tarPath);
      } else {
        throw new Error('File already exists');
      }
    }

    if (tgzOriginPath !== tarPath) {
      yield _fsExtra().default.move(tgzOriginPath, tarPath);
    }

    return response;
  });
  return _npmPack.apply(this, arguments);
}